@@ -1,9 +1,9 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -12,13 +12,15 @@
 ##############################################################################
 """Cacheable object and cache management base classes.
 
-$Id: Cache.py 24764 2004-05-17 06:13:48Z philikon $
+$Id: Cache.py 68835 2006-06-24 22:45:20Z tseaver $
 """
 import time, sys
+import logging
+logger = logging.getLogger('Cache')
+
 import Globals
 from Globals import DTMLFile
 from Acquisition import aq_get, aq_acquire, aq_inner, aq_parent, aq_base
-from zLOG import LOG, WARNING
 from AccessControl import getSecurityManager
 from AccessControl.Role import _isBeingUsedAsAMethod
 from AccessControl import Unauthorized
@@ -94,6 +96,8 @@ class Cacheable:
           'ZCacheable_getManagerId',
           'ZCacheable_getManagerIds',
           'ZCacheable_configHTML',
+          'ZCacheable_isAMethod',
+          'ZCacheable_getManagerURL',
           )),
         (ChangeCacheSettingsPermission,
          ('ZCacheable_setManagerId',
@@ -193,8 +197,7 @@ class Cacheable:
                                    mtime_func, default)
                 return val
             except:
-                LOG('Cache', WARNING, 'ZCache_get() exception',
-                    error=sys.exc_info())
+                logger.warn('ZCache_get() exception', error=sys.exc_info())
                 return default
         return default
 
@@ -211,8 +214,7 @@ class Cacheable:
                 c.ZCache_set(ob, data, view_name, keywords,
                              mtime_func)
             except:
-                LOG('Cache', WARNING, 'ZCache_set() exception',
-                    error=sys.exc_info())
+                logger.warn('ZCache_set() exception', error=sys.exc_info())
 
     def ZCacheable_invalidate(self, view_name='', REQUEST=None):
         '''Called after a cacheable object is edited. Causes all
@@ -229,8 +231,7 @@ class Cacheable:
             except:
                 exc = sys.exc_info()
                 try:
-                    LOG('Cache', WARNING, 'ZCache_invalidate() exception',
-                        error=exc)
+                    logger.warn('ZCache_invalidate() exception', error=exc)
                     message = 'An exception occurred: %s: %s' % exc[:2]
                 finally:
                     exc = None
