@@ -4,7 +4,7 @@
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -17,6 +17,7 @@ $Id: BTreeFolder2.py,v 1.27 2004/03/17 22:49:25 urbanape Exp $
 """
 
 import sys
+import logging
 from cgi import escape
 from urllib import quote
 from random import randint
@@ -35,9 +36,16 @@ from OFS.Folder import Folder
 from AccessControl import getSecurityManager, ClassSecurityInfo
 from AccessControl.Permissions import access_contents_information, \
      view_management_screens
-from zLOG import LOG, INFO, ERROR, WARNING
 from Products.ZCatalog.Lazy import LazyMap
+from zope.event import notify
+from zope.app.container.contained import ObjectAddedEvent
+from zope.app.container.contained import ObjectRemovedEvent
+from zope.app.container.contained import notifyContainerModified
+from OFS.event import ObjectWillBeAddedEvent
+from OFS.event import ObjectWillBeRemovedEvent
+import OFS.subscribers
 
+LOG = logging.getLogger('BTreeFolder2')
 
 manage_addBTreeFolderForm = DTMLFile('folderAdd', globals())
 
@@ -184,8 +192,7 @@ class BTreeFolder2Base (Persistent):
                             % repr(key))
             return 1
         except AssertionError:
-            LOG('BTreeFolder2', WARNING,
-                'Detected damage to %s. Fixing now.' % path,
+            LOG.warn( 'Detected damage to %s. Fixing now.' % path,
                 error=sys.exc_info())
             try:
                 self._tree = OOBTree(self._tree)
@@ -194,11 +201,11 @@ class BTreeFolder2Base (Persistent):
                     mt_index[key] = OIBTree(value)
                 self._mt_index = mt_index
             except:
-                LOG('BTreeFolder2', ERROR, 'Failed to fix %s.' % path,
+                LOG.error('Failed to fix %s.' % path,
                     error=sys.exc_info())
                 raise
             else:
-                LOG('BTreeFolder2', INFO, 'Fixed %s.' % path)
+                LOG.info('Fixed %s.' % path)
             return 0
 
 
@@ -404,47 +411,60 @@ class BTreeFolder2Base (Persistent):
                                         'it is already in use.' % id)
 
 
-    def _setObject(self, id, object, roles=None, user=None, set_owner=1):
-        v=self._checkId(id)
-        if v is not None: id=v
+    def _setObject(self, id, object, roles=None, user=None, set_owner=1,
+                   suppress_events=False):
+        ob = object # better name, keep original function signature
+        v = self._checkId(id)
+        if v is not None:
+            id = v
 
         # If an object by the given id already exists, remove it.
         if self.has_key(id):
             self._delObject(id)
 
-        self._setOb(id, object)
-        object = self._getOb(id)
+        if not suppress_events:
+            notify(ObjectWillBeAddedEvent(ob, self, id))
+
+        self._setOb(id, ob)
+        ob = self._getOb(id)
 
         if set_owner:
-            object.manage_fixupOwnershipAfterAdd()
+            # TODO: eventify manage_fixupOwnershipAfterAdd
+            # This will be called for a copy/clone, or a normal _setObject.
+            ob.manage_fixupOwnershipAfterAdd()
 
             # Try to give user the local role "Owner", but only if
             # no local roles have been set on the object yet.
-            if hasattr(object, '__ac_local_roles__'):
-                if object.__ac_local_roles__ is None:
-                    user=getSecurityManager().getUser()
-                    if user is not None:
-                        userid=user.getId()
-                        if userid is not None:
-                            object.manage_setLocalRoles(userid, ['Owner'])
+            if getattr(ob, '__ac_local_roles__', _marker) is None:
+                user = getSecurityManager().getUser()
+                if user is not None:
+                    userid = user.getId()
+                    if userid is not None:
+                        ob.manage_setLocalRoles(userid, ['Owner'])
+
+        if not suppress_events:
+            notify(ObjectAddedEvent(ob, self, id))
+            notifyContainerModified(self)
+
+        OFS.subscribers.compatibilityCall('manage_afterAdd', ob, ob, self)
 
-        object.manage_afterAdd(object, self)
         return id
 
 
-    def _delObject(self, id, dp=1):
-        object = self._getOb(id)
-        try:
-            object.manage_beforeDelete(object, self)
-        except BeforeDeleteException, ob:
-            raise
-        except ConflictError:
-            raise
-        except:
-            LOG('Zope', ERROR, 'manage_beforeDelete() threw',
-                error=sys.exc_info())
+    def _delObject(self, id, dp=1, suppress_events=False):
+        ob = self._getOb(id)
+
+        OFS.subscribers.compatibilityCall('manage_beforeDelete', ob, ob, self)
+
+        if not suppress_events:
+            notify(ObjectWillBeRemovedEvent(ob, self, id))
+
         self._delOb(id)
 
+        if not suppress_events:
+            notify(ObjectRemovedEvent(ob, self, id))
+            notifyContainerModified(self)
+
 
     # Aliases for mapping-like access.
     __len__ = objectCount
