@@ -1,9 +1,9 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -20,6 +20,7 @@ Python code.
 __version__='$Revision: 1.56 $'[11:-2]
 
 import sys, os, traceback, re, marshal, new
+import logging
 from Globals import DTMLFile, MessageDialog, package_home
 import AccessControl, OFS, RestrictedPython
 from Acquisition import aq_parent
@@ -33,10 +34,11 @@ from AccessControl import getSecurityManager
 from OFS.History import Historical, html_diff
 from OFS.Cache import Cacheable
 from AccessControl.ZopeGuards import get_safe_globals, guarded_getattr
-from zLOG import LOG, ERROR, INFO, PROBLEM
 from zExceptions import Forbidden
 import Globals
 
+LOG = logging.getLogger('Script (Python)')
+
 # Track the Python bytecode version
 import imp
 Python_magic = imp.get_magic()
@@ -209,7 +211,7 @@ class PythonScript(Script, Historical, Cacheable):
             getattr(self, 'Script_magic', None) != Script_magic):
             global _log_complaint
             if _log_complaint:
-                LOG(self.meta_type, INFO, _log_complaint)
+                LOG.info(_log_complaint)
                 _log_complaint = 0
             # Changes here won't get saved, unless this Script is edited.
             body = self._body.rstrip()
@@ -330,6 +332,14 @@ class PythonScript(Script, Historical, Cacheable):
         if item is self:
             self._filepath = self.get_filepath()
 
+    def manage_beforeDelete(self, item, container):
+        # shut up deprecation warnings
+        pass
+
+    def manage_afterClone(self, item):
+        # shut up deprecation warnings
+        pass
+
     def get_filepath(self):
         return self.meta_type + ':' + '/'.join(self.getPhysicalPath())
 
@@ -430,7 +440,7 @@ class PythonScript(Script, Historical, Cacheable):
             else:
                 self._makeFunction()
         except:
-            LOG(self.meta_type, ERROR, 'write failed', error=sys.exc_info())
+            LOG.error('write failed', error=sys.exc_info())
             raise
 
     def manage_FTPget(self):
