@@ -1,9 +1,9 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -12,7 +12,7 @@
 ##############################################################################
 """ ZCatalog product
 
-$Id: ZCatalog.py 39977 2005-11-08 15:19:40Z chrisw $
+$Id: ZCatalog.py 69101 2006-07-12 13:10:01Z shh $
 """
 
 from warnings import warn
@@ -37,7 +37,6 @@ from Products.PluginIndexes.common.PluggableIndex \
      import PluggableIndexInterface
 from Products.PluginIndexes.interfaces import IPluggableIndex
 from Products.PluginIndexes.TextIndex import Splitter
-from zLOG import LOG
 from zope.interface import implements
 
 from Catalog import Catalog, CatalogError
@@ -47,7 +46,10 @@ from ProgressHandler import ZLogHandler
 from ZCatalogIndexes import ZCatalogIndexes
 
 
-LOG = logging.getLogger('Zope.ZCatalog')
+logger = logging.getLogger('Zope.ZCatalog')
+
+# BBB:  some products apparently import 'LOG' from here!  Gone in 2.10.
+LOG = logger
 
 manage_addZCatalogForm=DTMLFile('dtml/addZCatalog',globals())
 
@@ -232,7 +234,7 @@ class ZCatalog(Folder, Persistent, Implicit):
 
             for url in urls:
                 obj = self.resolve_path(url)
-                if not obj:
+                if not obj and hasattr(self, 'REQUEST'):
                     obj = self.resolve_url(url, REQUEST)
                 if obj is not None:
                     self.catalog_object(obj, url)
@@ -296,8 +298,7 @@ class ZCatalog(Folder, Persistent, Implicit):
 
             p = paths[i]
             obj = self.resolve_path(p)
-            
-            if not obj and hasattr(self, 'REQUEST'):
+            if not obj:
                 obj = self.resolve_url(p, self.REQUEST)
             if obj is not None:
                 try:
@@ -305,8 +306,8 @@ class ZCatalog(Folder, Persistent, Implicit):
                 except ConflictError:
                     raise
                 except:
-                    LOG.error('Recataloging object at %s failed' % p,
-                              exc_info=sys.exc_info())
+                    logger.error('Recataloging object at %s failed' % p,
+                                 exc_info=sys.exc_info())
 
         if pghandler: pghandler.finish()
 
@@ -482,11 +483,10 @@ class ZCatalog(Folder, Persistent, Implicit):
             name = (name,)
 
         paths = self._catalog.uids.keys()
-        num_paths = len(paths)   # inefficient        
 
         i = 0
         if pghandler:
-            pghandler.init('reindexing %s' % name, num_paths)
+            pghandler.init('reindexing %s' % name, len(paths))
 
         for p in paths:
             i+=1
@@ -496,8 +496,8 @@ class ZCatalog(Folder, Persistent, Implicit):
             if obj is None:
                 obj = self.resolve_url(p, REQUEST)
             if obj is None:
-                LOG.error('reindexIndex could not resolve '
-                          'an object from the uid %r.' % p)
+                logger.error('reindexIndex could not resolve '
+                             'an object from the uid %r.' % p)
             else:
                 # don't update metadata when only reindexing a single
                 # index via the UI
@@ -911,7 +911,7 @@ class ZCatalog(Folder, Persistent, Implicit):
         self.pgthreshold = pgthreshold
         if RESPONSE:
             RESPONSE.redirect(
-                URL1 + '/manage_main?manage_tabs_message=Catalog%20Changed')
+                URL1 + '/manage_catalogAdvanced?manage_tabs_message=Catalog%20Changed')
 
     def _getProgressThreshold(self):
         if not hasattr(self, 'pgthreshold'):
@@ -926,9 +926,7 @@ class ZCatalog(Folder, Persistent, Implicit):
            classes. 
         """
 
-        LOG.info('Start migration of indexes for %s' % self.absolute_url(1))
-        
-        reindex_ids = []
+        logger.info('Start migration of indexes for %s' % self.absolute_url(1))
 
         for idx in self.Indexes.objectValues():
             bases = [str(name) for name in idx.__class__.__bases__]
@@ -945,7 +943,7 @@ class ZCatalog(Folder, Persistent, Implicit):
             if found:
                 idx_type = idx.meta_type
                 idx_id = idx.getId()
-                LOG.info('processing index %s' % idx_id)
+                logger.info('processing index %s' % idx_id)
 
                 indexed_attrs = getattr(idx, 'indexed_attrs', None)
 
@@ -963,14 +961,12 @@ class ZCatalog(Folder, Persistent, Implicit):
                 if idx.meta_type == 'DateRangeIndex':
                     setattr(new_idx, '_since_field',  since_field)
                     setattr(new_idx, '_until_field', until_field)
-                reindex_ids.append(idx_id)
-        
-        if reindex_ids:
-            LOG.info('Reindexing %s' % ', '.join(reindex_ids))
-            self.manage_reindexIndex(reindex_ids, REQUEST)
+
+                self.manage_reindexIndex(idx_id, REQUEST)
 
         self._migrated_280 = True
-        LOG.info('Finished migration of indexes for %s' % self.absolute_url(1))
+        logger.info('Finished migration of indexes for %s'
+                    % self.absolute_url(1))
 
         if RESPONSE:
             RESPONSE.redirect( URL1 +
