@@ -12,7 +12,7 @@
 ##############################################################################
 '''CGI Response Output formatter
 
-$Id: HTTPResponse.py 68778 2006-06-21 16:47:12Z tseaver $'''
+$Id: HTTPResponse.py 68779 2006-06-21 16:48:26Z tseaver $'''
 __version__ = '$Revision: 1.81 $'[11:-2]
 
 import types, os, sys, re
@@ -334,13 +334,18 @@ class HTTPResponse(BaseResponse):
                 self.body = body
 
 
+        isHTML = self.isHTML(self.body)
         if not self.headers.has_key('content-type'):
-            isHTML = self.isHTML(self.body)
             if isHTML:
-                c = 'text/html'
+                c = 'text/html; charset=%s' % default_encoding
             else:
-                c = 'text/plain'
+                c = 'text/plain; charset=%s' % default_encoding
             self.setHeader('content-type', c)
+        else:
+            c = self.headers['content-type']
+            if not 'charset=' in  c:
+                c = '%s; charset=%s' % (c, default_encoding)                
+                self.setHeader('content-type', c)
 
         # Some browsers interpret certain characters in Latin 1 as html
         # special characters. These cannot be removed by html_quote,
@@ -441,11 +446,18 @@ class HTTPResponse(BaseResponse):
                                               r')(?:(?:\s*;)|\Z)',
                                               re.IGNORECASE)):
         # Encode the Unicode data as requested
+
         if self.headers.has_key('content-type'):
             match = charset_re.match(self.headers['content-type'])
             if match:
                 encoding = match.group(1)
                 return body.encode(encoding)
+            else:
+
+                ct = self.headers['content-type']
+                if ct.startswith('text/') or ct.startswith('application/'):
+                    self.headers['content-type'] = '%s; charset=%s' % (ct, default_encoding)
+
         # Use the default character encoding
         return body.encode(default_encoding,'replace')
 
@@ -600,18 +612,9 @@ class HTTPResponse(BaseResponse):
     def _error_html(self,title,body):
         # XXX could this try to use standard_error_message somehow?
         return ("""\
-<table border="0" width="100%">
-<tr valign="top">
-
-<td width="10%" align="center">
-&nbsp;
-</td>
-
-<td width="90%">
   <h2>Site Error</h2>
   <p>An error was encountered while publishing this resource.
-  </p>""" + \
-  """
+  </p>
   <p><strong>%s</strong></p>
 
   %s""" %(title,body) + \
@@ -628,14 +631,12 @@ class HTTPResponse(BaseResponse):
   </ul>
 
   <p>For more detailed information about the error, please
-  refer to error log.
+  refer to the error log.
   </p>
 
   <p>If the error persists please contact the site maintainer.
   Thank you for your patience.
-  </p>
-</td></tr>
-</table>""")
+  </p>""")
 
 
     def notFoundError(self,entry='Unknown'):
