@@ -11,9 +11,13 @@
 ##############################################################################
 """HTTP DELETE verb
 
-$Id: delete.py 27079 2004-08-12 19:45:55Z srichter $
+$Id: delete.py 40824 2005-12-16 18:58:49Z mgedmin $
 """
+__docformat__ = 'restructuredtext'
+
 from zope.app.filerepresentation.interfaces import IWriteDirectory
+from zope.app.publication.http import MethodNotAllowed
+
 
 class DELETE(object):
     """Delete handler for all objects
@@ -32,7 +36,9 @@ class DELETE(object):
 
 
         # Get a "directory" surrogate for the container
-        dir = IWriteDirectory(container)
+        dir = IWriteDirectory(container, None)
+        if dir is None:
+            raise MethodNotAllowed(self.context, self.request)
 
         del dir[name]
 
