@@ -15,7 +15,7 @@
 
 See README.txt.
 
-$Id: config.py 27081 2004-08-12 19:56:31Z srichter $
+$Id: config.py 68962 2006-07-04 01:26:28Z tseaver $
 """
 __docformat__ = 'restructuredtext'
 import os.path
@@ -25,6 +25,7 @@ import sets
 import zope.schema
 
 from keyword import iskeyword
+import zope.deprecation
 from zope.configuration.exceptions import ConfigurationError
 from zope.configuration.interfaces import IConfigurationContext
 from zope.configuration.interfaces import IGroupingContext
@@ -92,6 +93,7 @@ class ConfigurationContext(object):
     def __init__(self):
         super(ConfigurationContext, self).__init__()
         self._seen_files = sets.Set()
+        self._features = sets.Set()
 
     def resolve(self, dottedname):
         """Resolve a dotted name to an object
@@ -105,10 +107,13 @@ class ConfigurationContext(object):
         1
         >>> c.resolve('zope.interface') is zope.interface
         1
-        >>> c.resolve('zope.configuration.eek')
+
+        >>> c.resolve('zope.configuration.eek') #doctest: +NORMALIZE_WHITESPACE
         Traceback (most recent call last):
         ...
-        ConfigurationError: Module zope.configuration has no global eek
+        ConfigurationError:
+        ImportError: Module zope.configuration has no global eek
+
         >>> c.resolve('.config.ConfigurationContext')
         Traceback (most recent call last):
         ...
@@ -178,9 +183,11 @@ class ConfigurationContext(object):
         try:
             mod = __import__(mname, *_import_chickens)
         except ImportError, v:
-            raise ConfigurationError, (
-                "Couldn't import %s, %s" % (mname, v)
-                ), sys.exc_info()[2]
+            if sys.exc_info()[2].tb_next is not None:
+                # ImportError was caused deeper
+                raise
+            raise ConfigurationError(
+                "ImportError: Couldn't import %s, %s" % (mname, v))
 
         if not oname:
             # see not mname case above
@@ -188,30 +195,21 @@ class ConfigurationContext(object):
 
 
         try:
-            return getattr(mod, oname)
+            zope.deprecation.__show__.off()
+            obj = getattr(mod, oname)
+            zope.deprecation.__show__.on()
+            return obj
         except AttributeError:
+            zope.deprecation.__show__.on()
             # No such name, maybe it's a module that we still need to import
             try:
                 return __import__(mname+'.'+oname, *_import_chickens)
             except ImportError:
-
-                # We need to try to figure out what module the import
-                # error is complaining about.  If the import failed
-                # due to a failure to import some other module
-                # imported by the module we are importing, we want to
-                # know it. Unfortunately, the value of the import
-                # error is just a string error message. :( We can't
-                # pull it apart directly to see what module couldn't
-                # be imported. The only thing we can really do is to
-                # try to "screen scrape" the error message:
-
-                if str(sys.exc_info()[1]).find(oname) < 0:
-                    # There seems to have been an error further down,
-                    # so reraise the exception so as not to hide it.
+                if sys.exc_info()[2].tb_next is not None:
+                    # ImportError was caused deeper
                     raise
-
-                raise ConfigurationError("Module %s has no global %s"
-                                         % (mname, oname))
+                raise ConfigurationError(
+                    "ImportError: Module %s has no global %s" % (mname, oname))
 
     def path(self, filename):
         """
@@ -228,7 +226,7 @@ class ConfigurationContext(object):
         >>> import zope.configuration
         >>> c.package = zope.configuration
         >>> import os
-        >>> d = os.path.split(zope.configuration.__file__)[0]
+        >>> d = os.path.dirname(zope.configuration.__file__)
         >>> c.path("y/z") == d + os.path.normpath("/y/z")
         1
         >>> c.path("y/./z") == d + os.path.normpath("/y/z")
@@ -251,7 +249,8 @@ class ConfigurationContext(object):
             if self.package is None:
                 basepath = os.getcwd()
             else:
-                basepath = os.path.split(self.package.__file__)[0]
+                basepath = os.path.dirname(self.package.__file__)
+                basepath = os.path.abspath(basepath)
             self.basepath = basepath
 
         return os.path.join(basepath, filename)
@@ -276,7 +275,7 @@ class ConfigurationContext(object):
         >>> import zope.configuration
         >>> c.package = zope.configuration
         >>> import os
-        >>> d = os.path.split(zope.configuration.__file__)[0]
+        >>> d = os.path.dirname(zope.configuration.__file__)
         >>> c.checkDuplicate('bar.zcml')
         >>> try:
         ...   c.checkDuplicate(d + os.path.normpath('/bar.zcml'))
@@ -310,7 +309,7 @@ class ConfigurationContext(object):
         >>> import zope.configuration
         >>> c.package = zope.configuration
         >>> import os
-        >>> d = os.path.split(zope.configuration.__file__)[0]
+        >>> d = os.path.dirname(zope.configuration.__file__)
         >>> c.processFile('bar.zcml')
         True
         >>> c.processFile('bar.zcml')
@@ -323,8 +322,6 @@ class ConfigurationContext(object):
         self._seen_files.add(path)
         return True
 
-
-
     def action(self, discriminator, callable=None, args=(), kw={}, order=0):
         """Add an action with the given discriminator, callable and arguments
 
@@ -365,7 +362,7 @@ class ConfigurationContext(object):
 
         Finally, we can add an order argument to crudely control the order
         of execution:
-        
+
         >>> c.action(None, order=99999)
         >>> c.actions[-1]
         (None, None, (), {}, ('foo.zcml',), '?', 99999)
@@ -383,6 +380,35 @@ class ConfigurationContext(object):
 
         self.actions.append(action)
 
+    def hasFeature(self, feature):
+        """Check whether a named feature has been provided.
+
+        Initially no features are provided
+
+        >>> c = ConfigurationContext()
+        >>> c.hasFeature('onlinehelp')
+        False
+
+        You can declare that a feature is provided
+
+        >>> c.provideFeature('onlinehelp')
+
+        and it becomes available
+
+        >>> c.hasFeature('onlinehelp')
+        True
+
+        """
+        return feature in self._features
+
+    def provideFeature(self, feature):
+        """Declare thata named feature has been provided.
+
+        See `hasFeature` for examples.
+        """
+        self._features.add(feature)
+
+
 class ConfigurationAdapterRegistry(object):
     """Simple adapter registry that manages directives as adapters
 
@@ -576,23 +602,24 @@ class ConfigurationMachine(ConfigurationAdapterRegistry, ConfigurationContext):
 
 
         """
-        for action in resolveConflicts(self.actions):
-            (discriminator, callable, args, kw, includepath, info, order
-             ) = expand_action(*action)
-            if callable is None:
-                continue
-            try:
-                callable(*args, **kw)
-            except:
-                if testing:
+        try:
+            for action in resolveConflicts(self.actions):
+                (discriminator, callable, args, kw, includepath, info, order
+                 ) = expand_action(*action)
+                if callable is None:
+                    continue
+                try:
+                    callable(*args, **kw)
+                except (KeyboardInterrupt, SystemExit):
                     raise
-                t, v = sys.exc_info()[:2]
-                v = ConfigurationExecutionError(t, v, info)
-                t = ConfigurationExecutionError
-                raise t, v, sys.exc_info()[2]
-
-        if clear:
-            del self.actions[:]
+                except:
+                    if testing:
+                        raise
+                    t, v, tb = sys.exc_info()
+                    raise ConfigurationExecutionError(t, v, info), None, tb
+        finally:
+            if clear:
+                del self.actions[:]
 
 
 class ConfigurationExecutionError(ConfigurationError):
@@ -994,16 +1021,10 @@ class GroupingContextDecorator(ConfigurationContext):
 ##############################################################################
 # Directive-definition
 
-class DirectiveSchema(fields.GlobalObject):
+class DirectiveSchema(fields.GlobalInterface):
     """A field that contains a global variable value that must be a schema
     """
 
-    def _validate(self, value):
-        super(fields.GlobalObject, self)._validate(value)
-        if not IInterface.providedBy(value):
-            raise WrongType(value)
-
-
 class IDirectivesInfo(Interface):
     """Schema for the ``directives`` directive
     """
@@ -1049,13 +1070,12 @@ class IFullInfo(IDirectiveInfo):
         description = u"The dotted name of the directive handler",
         )
 
-    usedIn = fields.GlobalObject(
+    usedIn = fields.GlobalInterface(
         title = u"The directive types the directive can be used in",
         description = (u"The interface of the directives that can contain "
                        u"the directive"
                        ),
         default = IConfigurationContext,
-        value_type = zope.schema.InterfaceField(),
         )
 
 class IStandaloneDirectiveInfo(IDirectivesInfo, IFullInfo):
@@ -1107,6 +1127,7 @@ def defineSimpleDirective(context, name, schema, handler,
 
     def factory(context, data, info):
         return SimpleStackItem(context, handler, info, schema, data)
+    factory.schema = schema
 
     context.register(usedIn, name, factory)
     context.document(name, schema, usedIn, handler, context.info)
@@ -1166,6 +1187,7 @@ def defineGroupingDirective(context, name, schema, handler,
         newcontext = handler(context, **args)
         newcontext.info = info
         return GroupingStackItem(newcontext)
+    factory.schema = schema
 
     context.register(usedIn, name, factory)
     context.document(name, schema, usedIn, handler, context.info)
@@ -1186,6 +1208,7 @@ class ComplexDirectiveDefinition(GroupingContextDecorator, dict):
 
         def factory(context, data, info):
             return ComplexStackItem(self, context, data, info)
+        factory.schema = self.schema
 
         self.register(self.usedIn, (self.namespace, self.name), factory)
         self.document((self.namespace, self.name), self.schema, self.usedIn,
@@ -1197,6 +1220,44 @@ def subdirective(context, name, schema):
                      context.info, context.context)
     context.context[name] = schema, context.info
 
+##############################################################################
+# Features
+
+class IProvidesDirectiveInfo(Interface):
+    """Information for a <meta:provides> directive"""
+
+    feature = zope.schema.TextLine(
+        title = u"Feature name",
+        description = u"""The name of the feature being provided
+
+        You can test available features with zcml:condition="have featurename".
+        """,
+        )
+
+def provides(context, feature):
+    """Declare that a feature is provided in context.
+
+    >>> c = ConfigurationContext()
+    >>> provides(c, 'apidoc')
+    >>> c.hasFeature('apidoc')
+    True
+
+    Spaces are not allowed in feature names (this is reserved for providing
+    many features with a single directive in the futute).
+
+    >>> provides(c, 'apidoc onlinehelp')
+    Traceback (most recent call last):
+      ...
+    ValueError: Only one feature name allowed
+
+    >>> c.hasFeature('apidoc onlinehelp')
+    False
+
+    """
+    if len(feature.split()) > 1:
+        raise ValueError("Only one feature name allowed")
+    context.provideFeature(feature)
+
 
 ##############################################################################
 # Argument conversion
@@ -1321,8 +1382,8 @@ def toargs(context, schema, data):
             try:
                 args[str(name)] = field.fromUnicode(s)
             except zope.schema.ValidationError, v:
-                raise ConfigurationError, (
-                    "Invalid value for", n, str(v)), sys.exc_info()[2]
+                raise ConfigurationError(
+                    "Invalid value for", n, str(v)), None, sys.exc_info()[2]
         elif field.required:
             # if the default is valid, we can use that:
             default = field.default
@@ -1569,5 +1630,12 @@ def _bootstrap(context):
             schema="zope.configuration.config.IDirectiveInfo"
             )
 
-    import zope.configuration.backward
-    zope.configuration.backward.bootstrap(context)
+    # meta:provides
+    context((metans, 'directive'),
+            info,
+            name='provides',
+            namespace=metans,
+            handler="zope.configuration.config.provides",
+            schema="zope.configuration.config.IProvidesDirectiveInfo"
+            )
+
